"""which module to find executables"""
from os import X_OK, access, environ, name as osname
from os.path import abspath, join as pjoin

def which(prog):
	"""which function like the linux 'which' program"""
	delim = ';' if osname == 'nt' else ':'
	for path in environ['PATH'].split(delim):
		if access(pjoin(path, prog), X_OK):
			return pjoin(abspath(path), prog)
	return ''



"""system lib init for pwclip"""
from system.android import adbout, adbenter

from system.clips import clips, copy, paste

from system.user import userfind, whoami

from system.path import absrelpath, filetime, setfiletime, filerotate

from system.random import randin, random

from system.which import which

from system.dictreplace import dictreplace

from system.envconf import envconf

from system.xlib import xinput, xnotify, xyesno, xmsgok, xgetpass, xkbid



"""x notification function"""
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
from sys import argv
from os.path import basename
try:
	from pynotifier import Notification
	#import gi
	#gi.require_version('Notify', '0.7')
	#from gi.repository import Notify as xnote
except (AttributeError, ImportError, ValueError) as err:
	def xnotify(msg, *_): """xnotify faker function""" ;return
else:
	def xnotify(msg, name=basename(argv[0]), wait=5):
		"""
		disply x notification usually in the upper right corner of the display
		"""
		try:
			Notification(title=name, description=msg, duration=wait).send()
		except (NameError, RuntimeError):
			pass



"""xkbid - x keyboard id"""
# -*- encoding: utf-8 -*-

from subprocess import Popen, PIPE, DEVNULL

def xkbid():
	o, e = Popen('xinput', stdout=PIPE, stderr=PIPE, shell=True).communicate()
	if o:
		o = o.decode()
	if e:
		e = e.decode()
		print(e)
	ln = [l.split()[-4] for l in o.split('\n') if 'keyboard' in l.lower()][-1]
	return int(ln.split('=')[1])



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
gpgtool module
"""
from os import path, environ, remove, walk, name as osname
try:
	from os import uname
except ImportError:
	# windumb faker function
	def uname(): return [0][environ['COMPUTERNAME']]
try:
	from os import chmod
except ImportError:
	# windumb faker function
	def chmod(*_): return

from shutil import move

from re import search

from getpass import getpass

from tkinter import TclError

from gnupg import GPG

import wget

try:
	import readline
except ImportError:
	pass

# local imports
from colortext import blu, yel, grn, bgre, tabd, abort, error, fatal

from system import xyesno, xgetpass, xmsgok, xinput, xnotify, userfind, which

from executor import Command

class DecryptError(Exception):
	pass
class EncryptionError(Exception):
	pass
class SignatureError(Exception):
	pass

class GPGTool(Command):
	"""
	gnupg wrapper-wrapper :P
	although the gnupg module is quite handy and the functions are pretty and
	useable i need some modificated easing functions to be able to make the
	main code more easy to understand by wrapping multiple gnupg functions to
	one - also i can prepare some program related stuff in here
	"""
	sh_ = True
	dbg = None
	dtg = True
	vrb = None
	gui = None
	frc = None
	iac = None
	sig = True
	dtg = True
	__c = 0
	__ppw = None
	homedir = path.join(path.expanduser('~'), '.gnupg')
	if 'GNUPGHOME' in environ.keys():
		homedir = environ['GNUPGHOME'].strip()
	__bin = 'gpg'
	if osname == 'nt':
		homedir = path.join(
            path.expanduser('~'), 'AppData', 'Roaming', 'gnupg')
		__bin = 'gpg.exe'
	_binary = which(__bin)
	_gpgselfcheck = None
	_keyserver = ''
	_gpgselfcheck = None
	kginput = {}
	recvs = []
	environ['LC_MESSAGES'] = 'C'
	if 'RECIPIENTS' in environ.keys():
		recvs = environ['RECIPIENTS'].split(' ')
	gpgkey = ''
	if 'GPGKEY' in environ.keys():
		gpgkey = environ['GPGKEY']
	cardkey = ''
	if 'CARDKEY' in environ.keys():
		cardkey = environ['CARDKEY']
	pwdmsg = 'enter passphrase: '
	def __init__(self, *args, **kwargs):
		"""gpgtool init function"""
		args = ['sh_'] + [a for a in args]
		for arg in args:
			setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(GPGTool.__mro__))
			print(bgre(tabd(GPGTool.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		Command.__init__(self, *args, **kwargs)
		if osname == 'nt' and not which('gpg.exe'):
			if not xyesno('mandatory gpg4win not found! Install it?'):
				raise RuntimeError('cannot continue without gpg4win')
			import wget
			src = 'https://files.gpg4win.org/gpg4win-latest.exe'
			trg = path.join(environ['TEMP'], 'gpg4win.exe')
			wget.download(src, out=trg)
			self.call(trg)
			remove(trg)

	@property                # keyring <str>
	def keyring(self):
		"""pubring getter (read-only)"""
		if self.binary.endswith('.exe'):
			return path.join(self.homedir, 'pubring.gpg')
		return path.join(self.homedir, 'pubring.kbx') \
			if self.binary.endswith('2') else path.join(
                self.homedir, 'pubring.gpg')

	@property                # secring <str>
	def secring(self):
		"""secring getter (read-only)"""
		if self.binary.endswith('.exe'):
			return path.join(self.homedir, 'secring.gpg')
		elif self.binary.endswith('2') and self.keyring.endswith('gpg'):
			return path.join(self.homedir, 'secring.gpg')
		return path.join(self.homedir, 'secring.kbx')

	@property                # binary <str>
	def binary(self):
		"""binary path getter"""
		return self._binary
	@binary.setter
	def binary(self, val):
		"""binary path setter"""
		self._binary = val

	@property                # _gpg_ <GPG>
	def _gpg_(self):
		"""gpg wrapper property"""
		opts = ['--batch']
		if osname != 'nt':
			opts = ['--pinentry-mode=loopback']
		if self.__c >= 1 and not self.__ppw:
			self.__ppw = self.passwd(False, self.gui, self.pwdmsg)
		__g = GPG(
            gnupghome=self.homedir, gpgbinary=self.binary,
            use_agent=True, options=opts, verbose=1 if self.dbg else 0)
		if osname != 'nt':
			__g.encoding = 'utf-8'
		return __g

	@staticmethod
	def passwd(rpt=False, gui=None, msg='enter passphrase: ',
          rptmsg='repeat that passphrase: '):
		"""password questioning method"""
		pas = getpass
		err = error
		if gui:
			pas = xgetpass
			err = xmsgok
		else:
			msg = blu(msg)
			rptmsg = blu(rptmsg)
		while True:
			if not rpt:
				return pas(msg)
			__pwd = pas(msg)
			if __pwd == pas(rptmsg):
				return __pwd
			err('passwords do not match')
		return False

	@staticmethod
	def __find(pattern, *vals):
		"""pattern matching method"""
		if not isinstance(pattern, str):
			raise error('pattern must be type string, got', '%s %s'%(
                type(pattern), pattern))
		for val in vals:
			if isinstance(val, (list, tuple)) and \
                  [v for v in val if pattern in v]:
				return True
			elif isinstance(val, dict) and pattern in val.values():
				return True
			elif pattern in val:
				return True
		return False

	@staticmethod
	def _gendefs(gui=False):
		user = environ['USERNAME'] if osname == 'nt' else environ['USER']
		host = environ['COMPUTERNAME'] if osname == 'nt' else uname()[1]
		kginput = {
                'name_real': user if len(user) >= 5 else '%s key'%user,
                'name_comment': '',
                'name_email': '%s@%s'%(user, host),
                'expire_date': 0,
                'key_type': 'RSA',
                'key_length': 4096,
                'subkey_type': 'RSA',
                'subkey_length': 4096}
		bea = False
		while True:
			echo = print
			ynq = ask = input
			_m = '%s\n%s\n%s [Y/n]\n'
			_g = grn('generating keys using:')
			_d = yel(tabd(kginput, 2))
			_o = grn('Is that OK?')
			if gui:
				echo = xmsgok
				ynq = xyesno
				ask = xinput
				_g = 'generating keys using:'
				_d = tabd(kginput, 2)
				_o = 'Is that OK?'
			msg = _m%(_g, _d, _o)
			try:
				yna = ynq(msg)
			except TclError as err:
				print(err)
				break
			if yna in ('n', False, None):
				msg = '%s [Y/n]'%grn('continue editing?')
				if gui:
					msg = 'abort editing?'
				yna = ynq(msg)
				if yna in ('n', False, None):
					break
			if yna in ('n', False, None):
				while True:
					for (k, v) in sorted(kginput.items()):
						nv = ask(
                            'enter new value for: "%s"\ncurrent value: "%s"\n' \
                            'enter "_" to unset or leave blank to use the ' \
                            ' preset value above\n'%(k, v))
						if nv == '_':
							del kginput[k]
							continue
						nv = v if not nv else nv
						kginput[k] = nv
					msg = _m%(_g, yel(tabd(kginput, 2)), _o)
					bea = ynq(msg)
					if bea is True or bea in ('n', ''):
						break
			elif yna in ('y', '', True):
				break
			if bea is not False:
				break
		return kginput

	def _listsecs(self, kllis, pattern=None):
		ksdics = {}
		kdic = {}
		for kl in kllis:
			t, dat = kl.split(':')[0], ':'.join(kl.split(':')[1:])
			if t == 'sec':
				if kdic:
					uid = kdic['uid']
					del kdic['uid']
					ksdics[uid] = kdic
					kdic = {}
				kdic[t] = dat
				continue
			kdic[t] = dat
		return ksdics

	def _listpubs(self, kllis, pattern=None):
		trust = kllis[0]
		del kllis[0]
		kls = []
		kdic = {'trust': trust}
		for kl in kllis:
			if kl.startswith('pub'):
				if kls:
					kdic[kl.strip()] = [k for k in kls]
					kls = []
				continue
			kls.append(kl)
		return kdic

	def gpglistkeys(self, pattern=None, secrets=None):
		opt = ' -k'
		lister = self._listpubs
		if secrets:
			lister = self._listsecs
			opt = ' -K'
		klstr = self.stdo(
			'%s --with-colons --status-fd 2 --fixed-list-mode%s'%(
				self.binary, opt))
		kdic = lister(klstr.split('\n'), pattern)
		if pattern:
			retdic = {}
			for (k, vs) in kdic.items():
				if pattern in k:
					retdic[k] = vs
					continue
				hit = False
				for v in vs:
					if pattern in v:
						hi = True
						break
				if hit:
					retdic[k] = vs
		return kdic

	def _gpgcheck(self):
		if self._gpgselfcheck:
			return
		gpgkey = self.cardkey if self.cardkey else self.gpgkey
		if not gpgkey:
			o = self.smcstatus()
			if o:
				for l in o.split('\n'):
					if l.startswith('sec'):
						gpgkey = str(l.split()[1]).split('/')[1]
		gpgkey = gpgkey if gpgkey else self.gpgkey
		self.sh_ = True
		getin = input
		quest = '%s [y/n]'%grn('could not authenticate, retry?')
		cmd = '%s --always-trust'%self.binary
		if self.gui:
			getin = xyesno
			quest = 'could not authenticate, retry?'
		pln = ' ^~ test ~^ '
		enc = self.stdo('%s -r %s -e'%(cmd, gpgkey), inputs=pln)
		dec = self.stdo('%s -d'%cmd, inputs=str(enc))
		if pln == dec:
			self._gpgselfcheck = True
			return True
		yesno = getin(quest)
		if str(yesno) not in ('true', 'y'):
			self.__c += 1
			return False
		return self._gpgcheck()

	def recvlist(self, crypt):
		keys = []
		out = self.stde('%s --list-only -v -d %s'%(
            self.binary ,crypt)).strip()
		_ks = []
		if out:
			_ks = [l.split(' ')[-1].strip() for l in out.split('\n') if l]
		for k in _ks:
			kv = self.findkey(k, typ='c')
			if not kv: continue
			keys.append('0x%s'%str(list(kv[0].keys())[0])[-16:])
		return keys

	def genkeys(self, **kginput):
		"""key-pair generator method"""
		if self.dbg:
			print(bgre(self.genkeys))
		kginput = kginput if kginput else self._gendefs(self.gui)
		kgmsg = '%s %s%s '%(
            blu('enter new password for'), yel(kginput['name_real']), blu(':'))
		if self.gui:
			echo = xmsgok
			kgmsg = 'enter new password for %s'%kginput['name_real']
		if 'passphrase' not in kginput.keys():
			kginput['passphrase'] = self.passwd(True, self.gui, kgmsg)
		return self._gpg_.gen_key(self._gpg_.gen_key_input(**kginput))

	def findkey(self, pattern='', **kwargs):
		"""key finder method"""
		typ = 'A' if 'typ' not in kwargs.keys() else kwargs['typ']
		sec = False if 'secret' not in kwargs.keys() else kwargs['secret']
		keys = []
		pattern = pattern if not pattern.startswith('0x') else pattern[2:]
		for key in self._gpg_.list_keys(secret=sec):
			if pattern and not self.__find(pattern, *key.values()):
				continue
			finger = key['fingerprint']
			subs = {}
			for (k, _) in key.items():
				if k == 'subkeys':
					#print(k)
					for sub in key[k]:
						#print(sub)
						_, typs, fin = sub
						#print(finger, typs)
						if typ == 'A' or (typ in typs):
							if typs not in subs.keys():
								subs[typs] = []
							subs[typs].append(fin)
			keys.append({finger: subs})
		return keys

	def smcstatus(self):
		if self.dbg:
			print(bgre(self.smcstatus))
		return self.stdo('%s --card-status'%self.binary)

	def smcfetch(self, urls):
		if self.dbg:
			print(bgre(self.smcfetch))
		if not urls:
			o = self.smcstatus()
			urls = []
			if o:
				for l in o.split('\n'):
					if l.startswith('URL of public key :'):
						urls = [':'.join(l.split(':')[1:])]
		eno = 0
		for url in urls:
			cmd = '%s --fetch-key %s'%(self.binary, url)
			e, o, n = self.oerc(cmd)
			#if o and self.vrb or self.dbg:
			if n != 0:
				eno = n
		return eno

	def keyimport(self, key):
		"""key from string import method"""
		if self.dbg:
			print(bgre('%s %s'%(self.keyimport, key)))
		return self._gpg_.import_keys(key)

	def _encryptwithkeystr(self, data, keystr, output):
		"""encrypt using given keystring method"""
		fingers = [
            r['fingerprint'] for r in self._gpg_.import_keys(keystr).results]
		return self._gpg_.encrypt(
            data, fingers, output=output)

	def _fingered(self, keys, typ='e'):
		fingers = []
		if not keys:
			return error('no keys received', keys, 'mode', typ)
		for key in keys:
			for (hsh, tyks) in key.items():
				if typ == 'c':
					fingers.append(hsh)
				else:
					for t in tyks.keys():
						if typ in t:
							fingers = fingers + tyks[t]
		return fingers

	def _signfromfile(self, path, finger, out):
		#print(path, finger, out, self.dtg)
		with open(path, 'rb') as cfh:
			return self._gpg_.sign_file(
                cfh, keyid=finger, detach=self.dtg, output=out)

	def sign(self, data, out=None):
		"""text encrypting method"""
		if self.dbg:
			print(bgre(self.sign))
		if not self.gpgkey:
			return error('key is needed for signing')
		if self.vrb:
			if out:
				print(blu('signing to'), yel(out))
		gpgkey = self.gpgkey
		if self.gpgkey and self.cardkey and self.gpgkey != self.cardkey:
			if self.vrb:
				error(
                    'card-key', self.cardkey, 'superseeds key set by config',
                    self.gpgkey, buzzword='WARNING')
			gpgkey = self.cardkey
		finger = self._fingered(self.findkey(gpgkey, typ='s'), 's')[0]
		#print(data, out)
		if path.isfile(data):
			sign = self._signfromfile(str(data), finger, out)
		else:
			sign = self._gpg_.sign(
                str(data), keyid=finger, detach=self.dtg, output=out)
		if path.isfile(out):
			chmod(out, 0o600)
		return sign

	def verify(self, sign, data=None):
		sig = False
		c = 0
		while c < 3:
			if path.isfile(sign):
				with open(data, 'rb') as sfh:
					sig = self._gpg_.verify_data(sign, sfh.read())
			else:
				sig = self._gpg_.verify(sign)
			if hasattr(sig, 'valid') and sig.valid:
				sig = True
			if sig:
				break
			c += 1
		return sig

	def encrypt(self, data, **kwargs):
		"""text encrypting method"""
		if self.dbg:
			print(bgre(self.encrypt))
		out = None if 'output' not in kwargs.keys() else kwargs['output']
		message = data
		if path.isfile(str(data)):
			with open(data, 'r') as cfh:
				message = str(cfh.read())
		recvs = self.recvs
		if 'recvs' in kwargs.keys():
			recvs = kwargs['recvs']
		if 'recipients' in kwargs.keys():
			recvs = kwargs['recipients']
		if self.vrb and out:
			print(blu('encrypting to'), yel(out))
		fingers = []
		for rec in recvs:
			fins = self._fingered(self.findkey(rec, typ='e'), 'e')
			fingers = fingers + fins if fins else []
		crypt = self._gpg_.encrypt(str(message), fingers, output=out)
		if not crypt.ok:
			raise EncryptionError('no valid encrypted object was returned')
		crypt = str(crypt)
		sign = False
		if self.sig:
			sig = '%s.sig'%out if out else None
			crypt = crypt if crypt else out
			sign = self.sign(crypt, sig)
		if out and path.isfile(out):
			chmod(out, 0o600)
		return crypt, sign

	def decrypt(self, data, **kwargs):
		"""text decrypting method"""
		if self.dbg:
			print(bgre(self.decrypt))
		out = None if not 'output' in kwargs.keys() else kwargs['output']
		sig = self.sig if 'sign' not in kwargs.keys() else kwargs['sign']
		message = data
		if path.isfile(data):
			with open(data, 'r') as cfh:
				message = str(cfh.read())
		sigerr = False
		if sig:
			sign = '%s.sig'%data
			if self.vrb:
				print(blu('validating'), yel(sign))
			signed = self.verify(sign, data)
			if not signed:
				sigerr = True
				yesno = False
				if not self.iac:
					yesno = True
					sigerr = False
					#error('skipping signature error in non-interactive mode')
				elif self.gui:
					yesno = xyesno(
                        'ERROR: signature could not be ' \
                        'verified\ncontinue anyways?')
				else:
					error('signature could not be verified')
					print(grn('continue anyways?'), '[y/N]')
					yesno = input()
					if yesno.lower() == 'y':
						yesno = True
				if not yesno:
					raise SignatureError('signature verification failed')
		__plain = None
		if self.vrb:
			print(blu('decrypting from'), yel(data))
		while self.__c < 4:
			__plain = self._gpg_.decrypt(
                message, output=out, always_trust=True, passphrase=self.__ppw)
			if __plain.ok:
				if sigerr:
					return [__plain, 'SIGERR']
				return [__plain, None]
			yesno = True
			if self.__c >= 3:
				yesno = False
				if self.gui:
					xnotify('too many wrong attempts')
				else:
					error('too many wrong attempts')
				raise DecryptError('cannot decrypt - too many wrong attempts')
			elif self.__c >= 1:
				yesno = False
				if self.gui:
					yesno = xyesno('decryption failed - try again?')
				else:
					yesno = True if str(input(
                        'decryption failed - retry? [Y/n]'
                        )).lower() in ('y', '') else False
			if yesno:
				mss = message.split('\n')
				raise DecryptError('%s failed to decrypt %s\n...\n%s'%(
                    self.decrypt, '\n'.join(mss[:3]), '\n'.join(mss[3:])))
			self.__c = self.__c+1
		return [__plain, 'KEYINTERRUPT']


class GPGSMTool(GPGTool):
	"""GPGSMTool class for compatibility to SSL keys/certificates"""
	dbg = False
	homedir = path.join(path.expanduser('~'), '.gnupg')
	__gsm = 'gpgsm'
	__ssl = 'openssl'
	if osname == 'nt':
		homedir = path.join(
            path.expanduser('~'), 'AppData', 'Roaming', 'gnupg')
		__gsm = 'gpgsm.exe'
		__ssl = 'openssl.exe'
	_gsmbin = which(__gsm)
	_sslbin = which(__ssl)
	sslcrt = ''
	sslkey = ''
	sslca = ''
	recvs = []
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if not self.recvs:
			if 'GPGKEYS' in environ.keys():
				self.recvs = environ['GPGKEYS'].split(' ')
			elif 'GPGKEY' in environ.keys():
				self.recvs = [environ['GPGKEY']]
		if self.sslcrt and self.sslkey:
			if osname == 'nt':
				raise RuntimeError(
                    'ssl import is currently not available for windows')
			self.sslimport(self.sslkey, self.sslcrt, self.sslca)
		if self.dbg:
			print(bgre(GPGSMTool.__mro__))
			print(bgre(tabd(GPGSMTool.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		GPGTool.__init__(self, *args, **kwargs)

	def sslimport(self, key, crt, ca):
		"""ssl key/cert importing method"""
		if self.dbg:
			print(bgre('%s key=%s crt=%s'%(self.sslimport, key, crt)))
		self.stdo('%s --import'%self._gsmbin, inputs=self.stdo(
            '%s pkcs12 -export -chain -CAfile %s -in %s -inkey %s'%(
                self._sslbin, ca, crt, key), b2s=False), b2s=False)

	def keylist(self, secret=False):
		"""key listing function"""
		if self.dbg:
			print(bgre(self.keylist))
		gsc = 'gpgsm -k'
		if secret:
			gsc = 'gpgsm -K'
		strs = self.stdo(gsc)
		keys = []
		if strs:
			strs = str('\n'.join(strs.split('\n')[2:])).split('\n\n')
			for ks in strs:
				if not ks: continue
				kid = str(ks.split('\n')[0].strip()).split(': ')[1]
				inf = [i.strip() for i in ks.split('\n')[1:]]
				key = {kid: {}}
				for i in inf:
					key[kid][i.split(': ')[0]] = i.split(': ')[1]
				keys.append(key)
		return keys

	def findkey(self, pattern=''):
		return self.keylist()

	def encrypt(self, message, **kwargs):
		"""text encrypting method"""
		if self.dbg:
			print(bgre(self.encrypt))
		recvs = self.recvs if 'recvs' not in kwargs.keys() else kwargs['recvs']
		if 'recipients' in kwargs.keys():
			recvs = kwargs['recipients']
		recvs = ''.join(['-r %s'%r for r in recvs])
		out = '' if 'output' not in kwargs.keys() else '-o %s'%kwargs['output']
		gsc = '%s -e --armor --disable-policy-checks --disable-crl-checks ' \
            '%s %s'%(self._gsmbin, out, recvs)
		__crypt = self.stdo(gsc, inputs=message.encode())
		if __crypt:
			return __crypt.decode()
		return False

	def decrypt(self, message, output=None):
		"""text decrypting method"""
		if self.dbg:
			print(bgre(self.decrypt))
		out = '' if not output else '-o %s'%'output'
		gsc = '%s -d %s'%(self._gsmbin, out)
		__plain = self.stdo(gsc, inputs=message)
		if __plain:
			return __plain
		return False



#!/usr/bin/python3
"""command module of executor"""
from os import access, environ, X_OK, name as osname
try:
	from os import getuid, getresuid
except ImportError:
	def getuid(): return 1000
	def getresuid(): return 1000

from os.path import abspath, join as pjoin

from sys import stdout as _stdout, stderr as _stderr

from subprocess import run, Popen, PIPE, TimeoutExpired

try:
	from subprocess import DEVNULL
except ImportError:
	DEVNULL = open('/dev/null')

try:
	from tkinter import StringVar, Button, Entry, Frame, Label, Tk, TclError
except ImportError:
	from Tkinter import StringVar, Button, Entry, Frame, Label, Tk, TclError
try:
	import readline
except ImportError:
	pass

_echo_ = _stdout.write
_puke_ = _stderr.write

class XInput(Frame):
	"""password clipping class for tkinter.Frame"""
	inp = None
	def __init__(self, master, message, exchange='', noop=None, noin=None):
		self.noop = noop
		self.noin = noin
		self.exchg = exchange
		self.message = message
		Frame.__init__(self, master)
		self.pack()
		self.inputwindow()
	def _enterexit(self, _=None):
		"""exit by saving challenge-response for input"""
		self.inp = True if not self.input else self.input.get()
		self.quit()
	def _breakexit(self, _=None):
		"""exit by saving challenge-response for input"""
		self.inp = None
		self.quit()
	def _exit(self, _=None):
		"""just exit (for ESC mainly)"""
		self.inp = False
		self.quit()
	def inputwindow(self):
		"""password input window creator"""
		self.lbl = Label(self, text=self.message)
		self.lbl.pack()
		okside = {}
		clside = {}
		self.ok = Button(self)
		self.ok.bind("<Control-c>", self._breakexit)
		self.ok.bind("<Escape>", self._exit)
		self.ok.bind("<Return>", self._enterexit)
		self.ok["text"] = "ok"
		self.ok["command"] =  self._enterexit
		self.ok.bind("<Return>", self._enterexit)
		self.input = False
		okside = {'side': 'bottom'}
		if not self.noin:
			self.input = StringVar()
			self.entry = Entry(self, show=self.exchg)
			self.entry.bind("<Return>", self._enterexit)
			self.entry.bind("<Control-c>", self._breakexit)
			self.entry.bind("<Escape>", self._exit)
			self.entry.focus_set()
			self.entry["textvariable"] = self.input
			self.entry.pack()
			okside = {'side': 'left'}
		if not self.noop:
			self.cl = Button(self)
			self.cl.bind("<Return>", self._enterexit)
			self.cl.bind("<Control-c>", self._breakexit)
			self.cl.bind("<Escape>", self._exit)
			self.cl["text"] = "cancel"
			self.cl["command"] = self._exit
			self.cl.pack({'side': 'right'})
			okside = {'side': 'left'}
		if self.noin:
			self.ok.focus_set()
		self.ok.pack(okside)

def xgetpass(message="input will not be displayed"):
	"""gui representing function"""
	try:
		root = Tk()
		pwc = XInput(root, message, exchange='*')
		_set_focus(root)
		pwc.lift()
		pwc.mainloop()
	except KeyboardInterrupt:
		root.destroy()
		raise KeyboardInterrupt
	try:
		root.destroy()
	except:
		pass
	finally:
		return pwc.inp

def _set_focus(window):
	"""os independent focus setter"""
	if osname == 'nt':
		window.after(1, lambda: window.focus_force())


class Command(object):
	"""Command class"""
	sh_ = True
	su_ = False
	gui = False
	vrb = False
	dbg = False
	_auth = False
	timeout = None
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, str(arg)):
				setattr(self, arg, True)
			elif hasattr(self, '%s_'%arg):
				setattr(self, '%s_'%arg, True)
		if self.dbg:
			cb = '\033[1;30m'
			ce = '\033[0m'
			if osname == 'nt':
				cb = ''
				ce = ''
			_echo_('%s%s\n  %s\n  %s'%(
                cb, str(Command.__mro__),
                '\n  '.join('%s = %s'%(k, v) for \
                    (k, v) in self.__dict__.items()), ce))

	@staticmethod
	def _sttyecho():
		run('stty echo', shell=True)

	@staticmethod
	def __which(prog):
		"""which function like the linux 'which' program"""
		delim = ';' if osname == 'nt' else ':'
		__path = ''
		for path in environ['PATH'].split(delim):
			if access(pjoin(path, prog), X_OK):
				__path = pjoin(abspath(path), prog)
		return __path

	@staticmethod
	def _str(commands):
		"""list/tuple to str converter"""
		#print(commands)
		return ' '.join(str(command) for command in list(commands))

	@staticmethod
	def __sucmd(sudobin, commands):
		if 'sudo' in commands[0]:
			del commands[0]
		if int(getuid()) != 0:
			commands.insert(0, sudobin)
		return commands

	def _list(self, commands):
		"""
		commands string to list converter assuming at least one part
		"""
		#print(commands)
		for cmd in list(commands):
			if cmd and max(len(c) for c in cmd if c) == 1 and len(cmd) >= 1:
				return list(commands)
			return self._list(list(cmd))

	def _sudo(self, commands=None):
		"""privilege checking function"""
		sudo = self.__which('sudo')
		if getuid() == 0:
			self._auth = True
			return commands if commands else True
		if not commands:
			if getuid() == 0: return True
			if int(run([sudo, '-v']).returncode) == 0:
				return True
			sucmds = None
			if self.gui:
				try:
					if int(run(
                          '%s -v'%sudo, timeout=0.5, shell=True).returncode) == 0:
						self._auth = True
				except TimeoutExpired:
					pass
				if not self._auth and int(run('%s -S -v'%sudo, shell=True,
                          input=str('%s\n'%xgetpass(
                          'enter sudo password')).encode()
                          ).returncode) == 0:
					self._auth = True
			elif int(run('%s -v'%sudo, shell=True).returncode) == 0:
				self._auth = True
		if self._auth:
			return self.__sucmd(sudo, commands)
		else:
			if not self._sudo():
				raise PermissionError('cannot execute privileged commands')
			return self.__sucmd(sudo, commands)
		#print(commands)
		#print(self.gui)
		#print(self._auth)
		return self._auth

	def __cmdprep(self, commands, func):
		commands = self._list(commands)
		if self.su_:
			try:
				commands = self._sudo(commands)
			finally:
				self._sttyecho()
		if self.sh_:
			commands = self._str(commands)
		if self.dbg:
			_echo_('\033[01;30m%s\n  `%s`\t{sh: %s, su: %s}\033[0m\n'%(
                func, commands, self.sh_, self.su_))
		return commands

	def run(self, *commands):
		"""just run the command and return the processes PID"""
		commands = self.__cmdprep(commands, self.run)
		return Popen(
            commands,
            stdout=DEVNULL, stderr=DEVNULL, shell=self.sh_).pid

	def call(self, *commands, stdout=True, stderr=True, inputs=None, b2s=None):
		"""
		default command execution
		prints STDERR, STDOUT and returns the exitcode
		"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		stderr = stderr if stderr else DEVNULL
		stdout = stdout if stdout else DEVNULL
		commands = self.__cmdprep(commands, self.call)
		return int(run(
            commands, shell=self.sh_, stdout=stdout,
            stderr=stderr, timeout=self.timeout, input=inputs).returncode)

	def stdx(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR and/or STDOUT"""
		commands = self.__cmdprep(commands, self.stdx)
		inputs = inputs.encode() if (inputs and b2s) else inputs
		prc = Popen(
		    commands, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		out, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and out:
			out = u'%s'%(out).decode()
		if b2s and err:
			err = u'%s'%(err).decode()
		return out, err

	def stdo(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDOUT only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.stdo)
		prc = Popen(
            commands, stdin=PIPE, stdout=PIPE, stderr=DEVNULL, shell=self.sh_)
		out, _ = prc.communicate(input=inputs, timeout=self.timeout)
		if b2s:
			out = u'%s'%(out).decode()
		return out

	def stde(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.stde)
		prc = Popen(
		    commands, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		_, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and err:
			err = u'%s'%(err).decode()
		return err

	def erno(self, *commands, inputs=None, b2s=True):
		"""command execution which returns the exitcode only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.erno)
		prc = Popen(commands, stdin=PIPE, stdout=DEVNULL,
            stderr=DEVNULL, shell=self.sh_)
		prc.communicate(timeout=self.timeout, input=inputs)
		return int(prc.returncode)

	def oerc(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.oerc)
		prc = Popen(
		    commands, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		out, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and out :
			out = u'%s'%(out).decode()
		if b2s and err:
			err = u'%s'%(err).decode()
		return out, err, int(prc.returncode)


cmmd = Command('sh')
sudo = Command('sh', 'su')

def sudofork(*args):
	"""sudo command fork wrapper function"""
	if getresuid() == 0:
		return 0
	eno = 0
	try:
		eno = int(sudo.call(args))
	except KeyboardInterrupt:
		_echo_('\033[34maborted by keystroke\033[0m\n')
		eno = 431
	return eno

if __name__ == '__main__':
	exit(1)



#!/usr/bin/env python3

import sys
import logging
#from rainbow_logging_handler import RainbowLoggingHandler
"""
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

#The background is set with 40 plus the number of the color, and the foreground with 30

#These are the sequences need to get colored ouput
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"

def formatter_message(message, use_color = True):
    if use_color:
        message = message.replace("$RESET", RESET_SEQ).replace("$BOLD", BOLD_SEQ)
    else:
        message = message.replace("$RESET", "").replace("$BOLD", "")
    return message

COLORS = {
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': BLUE,
    'CRITICAL': YELLOW,
    'ERROR': RED
}

class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color = True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        levelname = record.levelname
        if self.use_color and levelname in COLORS:
            levelname_color = COLOR_SEQ % (30 + COLORS[levelname]) + levelname + RESET_SEQ
            record.levelname = levelname_color
        return logging.Formatter.format(self, record)

And to use it, create your own Logger:

# Custom logger class with multiple destinations
class ColoredLogger(logging.Logger):
    FORMAT = "[$BOLD%(name)-20s$RESET][%(levelname)-18s]  %(message)s ($BOLD%(filename)s$RESET:%(lineno)d)"
    COLOR_FORMAT = formatter_message(FORMAT, True)
    def __init__(self, name):
        logging.Logger.__init__(self, name, logging.DEBUG)

        color_formatter = ColoredFormatter(self.COLOR_FORMAT)

        console = logging.StreamHandler()
        console.setFormatter(color_formatter)

        self.addHandler(console)
        return


logging.setLoggerClass(ColoredLogger)
"""




def logger(name, lvl='info'):
#	# 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'
#	lvls = {
#      'debug': logging.DEBUG,
#      'info': logging.INFO,
#      'warning': logging.WARNING,
#      'error': logging.ERROR,
#      'critical': logging.CRITICAL}
#	_logger = logging.getLogger(name)
#	_logger.setLevel(lvl if lvl in lvls.values() else lvls[lvl])
#	formats = logging.Formatter("[%(asctime)-19s]%(lineno)-6d:%(filename)s%(module)s:%(funcName)s: %(message)s")  # same as default
#	handler = logging.FileHandler(name)
#	_logging.basicConfig
#	handler.setFormatter(formats)
#	_logger.addHandler(handler)
	logging.basicConfig(format='[%(asctime)-19s]%(lineno)-6d:%(filename)s%(module)s:%(funcName)s: %(message)s', datefmt='%F.%T')
	return logging



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK
# -*- coding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""pwclip main program"""
# global & stdlib imports
try:
	from os import fork
except ImportError:
	def fork(): """fork faker function""" ;return 0

try:
	import readline
except ImportError:
	pass

from re import sub

from os import environ, path, remove, getpid, name as osname

from sys import exit, stdout

from argparse import ArgumentParser

import argcomplete
from argcomplete import autocomplete
from argcomplete.completers import FilesCompleter, ChoicesCompleter

from socket import gethostname as hostname

from time import sleep

from yaml import load, Loader

from getpass import getpass

# local relative imports
from colortext import bgre, bred, tabd, error, fatal, abort

from executor.executor import cmmd

from system import \
    absrelpath, copy, paste, xgetpass, \
    xmsgok, xyesno, xnotify, xinput, xkbid, \
    which, whoami, dictreplace, adbout

from pwclip.passcrypt import PassCrypt, lscrypt

from secrecy import ykchalres, yubikeys

from pwclip.__pkginfo__ import version, name as __me


def showclip(mode='cli', cliptime=None, clear=None):
	text = paste()
	if mode == 'cli':
		print(text, end='')
		stdout.flush()
		if clear:
			if cliptime:
				sleep(cliptime)
			print('\033c', end='')
	elif mode == 'ano':
		adbout(text)
	elif mode == 'gui':
		xnotify(text, __me)

def forkwaitclip(text, poclp, boclp, wait=3, out=None, enter=None):
	"""clipboard forking, after time resetting function"""
	if out:
		xnotify('paste', __me)
		if out == 'gui':
			sep = '"'
			if "'" in text and '"' in text:
				sep = '"'
				text = r'\"'.join(text.split('"'))
			elif "'" in text:
				sep = '"'
			elif '"' in text:
				sep = "'"
			if '$' in text:
				text = r'\$'.join(text.split('$'))
			#	text = sub(r'$', r'$', text)
			with open('/tmp/bla', 'w+') as tfh:
				tfh.write(fr'{sep}{text}{sep}')
			cmmd.stdo(fr'xvkbd -secure -no-keypad -delay 17 -text {sep}{text}{sep}')
		elif out == 'cli':
			stdout.write(r'%s'%text if not enter else r'%s\n'%text)
			stdout.flush()
		elif out == 'ano':
			adbout(text, enter)
			enter = False
	if enter:
		cmmd.call('%s -i %s "key Return"'%(which('xte'), xkbid()))
	copy(text, mode='pb')
	if fork() == 0:
		try:
			sleep(int(wait))
		finally:
			copy(poclp, mode='p')
			copy(boclp, mode='b')
	exit(0)

def __passreplace(pwlist):
	"""returnes a string of asterisk's as long as the password is"""
	__pwcom = ['*'*len(str(pwlist[0]))]
	if len(pwlist) > 1:
		__pwcom.append(pwlist[1])
	return __pwcom

def __dctpwreplace(pwdict):
	"""password => asterisk replacement function"""
	__pwdict = {}
	for (usr, ent) in pwdict.items():
		if isinstance(ent, dict):
			__pwdict[usr] = {}
			for (u, e) in ent.items():
				__pwdict[usr][u] = __passreplace(e)
		elif ent:
			__pwdict[usr] = __passreplace(ent)
	return __pwdict

def _envconf(srcdict):
	newdict = {}
	for (k, v) in srcdict.items():
		if k in environ.keys():
			newdict[v] = environ[k]
	return newdict

def _printpws_(pwdict, insecure=False):
	"""password printer with in/secure option"""
	if not insecure:
		pwdict = __dctpwreplace(pwdict)
	print(tabd(pwdict))
	exit(0)

def optpars(cfgs, mode, name):
	desc = 'pwclip - Multi functional password manager to temporarily ' \
           'save passphrases to your copy/paste buffers for easy and ' \
           'secure accessing your passwords. The following ' \
           'arguments might also be set by the config ' \
           '~/.config/%s.yaml file.'%name
	epic = 'the yubikey mode is compatible with the challenge-response ' \
           'feature of yubikeys only for now.'
	pars = ArgumentParser(description=desc, epilog=epic)
	pars.set_defaults(**cfgs)
	pars.add_argument(
        '--version',
        action='version', version='%(prog)s-v'+version)
	pars.add_argument(
        '-D', '--debug',
        dest='dbg', action='store_true', help='debugging mode')
	pars.add_argument(
        '-A', '--all',
        dest='aal', action='store_true',
        help='switch to all users entrys ("%s" only is default)'%cfgs['user'])
	pars.add_argument(
        '-E', '--enter',
        dest='ent', action='store_true',
        help='also enter newline when printing password (only useful with ' \
             '-o/-O)')
	pars.add_argument(
        '-O', '--android',
        dest='out', action='store_const', const='ano',
        help='print password to stdout of android device if one is connected')
	pars.add_argument(
        '-o', '--stdout',
        dest='out', action='store_const', const=mode,
        help='print password to stdout (insecure and not recommended)')
	pars.add_argument(
        '-e', '--expression',
        dest='rex', default="[a-zA-Z0-9\!$%%&/\(\)=\?\+#,\.-:]*:24", metavar='EXPRESSION:[LEN]',
        help='generate password using EXPRESSION to generate password of '
             'lenght LEN, either LEN is set or 24 is used - use with -g' \
             '(default is "[a-zA-Z0-9\!$%%&/\(\)=\?\+#,\.-:]*:24")')
	pars.add_argument(
        '-g', '--genpw',
        dest='gpw', action='store_true', help='randomly generate password' \
             ', only useful with -a (uses EXPRESSION)')
	pars.add_argument(
        '--show-clip',
        dest='scl', action='store_true',
        help='show current clipboard content ' \
             '(WARNING: insecure - if cliptime set ' \
             'to 0 cli mode does not clear the sreen)')
	pars.add_argument(
        '-s', '--show-passwords',
        dest='sho', action='store_true',
        help='show passwords when listing (replaced by "*" is default)')
	pars.add_argument(
        '-t',
        dest='time', default=3, metavar='seconds', type=int,
        help='time to wait before resetting clip (%s is default)'%cfgs['time'])
	gpars = pars.add_argument_group('gpg/ssl arguments')
	gpars.set_defaults(**cfgs)
	gpars.add_argument(
        '-k', '--key',
        dest='gpgkey', metavar='ID', type=str,
        help='gpg-key ID(s) to use for decryption/signing')
	gpars.add_argument(
        '-r', '--recipients',
        dest='recvs', metavar='"ID [ID ...]"',
        help='one ore more gpg-key ID(s) to use for ' \
             'encryption (strings seperated by spaces within "")')
	gpars.add_argument(
        '-u', '--user',
        dest='usr', metavar='USER', nargs='?' if mode == 'gui' else None,
        default=False if mode == 'gui' else cfgs['user'],
        help='query entrys only for USER (-A overrides, ' \
             '"%s" is default)'%cfgs['user'])
	gpars.add_argument(
        '-p', '--password',
        dest='pwd', default=None,
        help='enter password for add/change actions' \
             '(insecure & not recommended)')
	gpars.add_argument(
        '--comment',
        dest='com', default=None,
        help='enter comment for add/change actions')
	gpars.add_argument(
        '-x', '--x509',
        dest='gpv', action='store_const', const='gpgsm',
        help='force ssl compatible gpgsm mode - usually is autodetected ' \
             '(use --cert & --key for imports)')
	gpars.add_argument(
        '-C', '--cert',
        dest='sslcrt', metavar='SSL-Certificate',
        help='one-shot setting of SSL-Certificate')
	gpars.add_argument(
        '-K', '--ssl-key',
        dest='sslkey', metavar='SSL-Private-Key',
        help='one-shot setting of SSL-Private-Key')
	gpars.add_argument(
        '--ca', '--ca-cert',
        dest='sslca', metavar='SSL-CA-Certificate',
        help='one-shot setting of SSL-CA-Certificate')
	gpars.add_argument(
        '-P', '--passcrypt',
        dest='pcr', metavar='CRYPTFILE',
        default=path.expanduser('~/.passcrypt'),
        help='set location of CRYPTFILE to use as ' \
             'password store (~/.passcrypt is default)')
	gpars.add_argument(
        '-Y', '--yaml',
        dest='yml', metavar='YAMLFILE',
        default=path.expanduser('~/.pwd.yaml'),
        help='set location of YAMLFILE to read whole ' \
             'sets of passwords from a yaml file (~/.pwd.yaml is default)')
	ypars = pars.add_argument_group('yubikey arguments')
	ypars.set_defaults(**cfgs)
	ypars.add_argument(
        '-S', '--slot',
        dest='ysl', default=None, type=int, choices=(1, 2),
        help='set one of the two yubikey slots (only useful with -y)'
        ).completer = ChoicesCompleter((1, 2))
	ypars.add_argument(
        '-y', '--ykserial',
        nargs='?', dest='yks', metavar='SERIAL', default=False,
        help='switch to yubikey mode and optionally set ' \
		     'SERIAL of yubikey (autoselect serial and slot is default)')
	apars = pars.add_argument_group('action arguments')
	apars.set_defaults(**cfgs)
	apars.add_argument(
        '-a', '--add',
        dest='add', metavar='ENTRY', nargs='?' if mode == 'gui' else None,
        default=False, help='add ENTRY (password will be asked interactivly)')
	apars.add_argument(
        '-c', '--change',
        dest='chg', metavar='ENTRY', nargs='?' if mode == 'gui' else None,
        default=False, help='change ENTRY (password will be asked interactivly)')
	apars.add_argument(
        '-d', '--delete',
        dest='rms', metavar='ENTRY', nargs='?' if mode == 'gui' else '+',
        default=False, help='delete ENTRY(s) from the passcrypt list')
	apars.add_argument(
        '-l', '--list',
        dest='lst', metavar='PATTERN', nargs='?',
        default=False,
        help='pwclip an entry matching PATTERN if given ' \
             '- otherwise list all entrys')
	autocomplete(pars)
	args = pars.parse_args()
	return pars, args

def confpars(mode):
	"""pwclip command line opt/arg parsing function"""
	_me = path.basename(path.dirname(__file__))
	cfg = path.expanduser('~/.config/%s.cfg'%_me)
	cfgs = {
        'crypt': path.expanduser('~/.passcrypt'),
        'plain': path.expanduser('~/.pwd.yaml'),
        'time': 3,
        'binary': 'gpg' if osname == 'nt' else 'gpg.exe',
        'user': whoami(),
        }
	try:
		with open(cfg, 'r') as cfh:
			confs = dict(load(cfh.read(), Loader=Loader))
	except (TypeError, FileNotFoundError):
		confs = {}
	cfgmap = {
        'gpg': {'recipients': 'recvs', 'key': 'gpgkey', 'delkey': True},
        'yubikey': {'slot': 'ykslot', 'seerial': 'ykser', 'delkey': True}}
	envmap = {
        'GPGKEY': 'gpgkey',
        'RECIPIENTS': 'recvs',
        'PWCLIPTIME': 'time',
        'YKSERIAL': 'ykser',
        'USER': 'usr',
        'USERNAME': 'usr',
        'YKSLOT': 'ykslot'}
	confs = dictreplace(confs, cfgmap)
	for (k, v) in confs.items():
		cfgs[k] = v
	envs = _envconf(envmap)
	for (k, v) in envs.items():
		cfgs[k] = v
	cfgs['binary'] = 'gpg2'
	if osname == 'nt':
		cfgs['binary'] = 'gpg'
	if 'crypt' not in cfgs.keys():
		cfgs['crypt'] = path.expanduser('~/.passcrypt')
	elif 'crypt' in cfgs.keys() and cfgs['crypt'].startswith('~'):
		cfgs['crypt'] = path.expanduser(cfgs['crypt'])
	if 'plain' not in cfgs.keys():
		cfgs['plain'] = path.expanduser('~/.pwd.yaml')
	elif 'plain' in cfgs.keys() and cfgs['plain'].startswith('~'):
		cfgs['plain'] = path.expanduser(cfgs['plain'])
	pars, args = optpars(cfgs, mode, _me)
	pargs = [a for a in [
        'aal' if args.aal else None,
        'dbg' if args.dbg else None,
        'ent' if args.ent else None,
        'gsm' if args.gpv else None,
        'gui' if mode == 'gui' else None,
        'sho' if args.sho else None] if a]
	__bin = 'gpg2'
	if args.gpv:
		__bin = args.gpv
	if osname == 'nt':
		__bin = 'gpgsm.exe' if args.gpv else 'gpg.exe'
	pkwargs = {}
	pkwargs['binary'] = __bin
	pkwargs['sslcrt'] = args.sslcrt
	pkwargs['sslkey'] = args.sslkey
	if args.gpw:
		_genpwrex = args.rex
		pargs.append('rnd')
		if args.rex is False:
			getrex = input
			msg = 'enter regular expression for new password:'
			err = 'aborted due to empty input'
			if mode == 'gui':
				getrex = xinput
				msg = 'enter regular expression for new password'
				err = 'aborted due to empty input'
			_genpwrex = getrex('enter regular expression for new password')
			if not _genpwrex:
				xnotify('aborted due to empty input', __me)
				exit(1)
		genpwrex = _genpwrex
		genpwlen = 24
		if ':' in genpwrex:
			genpwrex, genpwlen = \
                ''.join(str(genpwrex).split(':')[:-1]), \
                str(genpwrex).split(':')[-1]
		pkwargs['genpwrex'] = genpwrex
		pkwargs['genpwlen'] = genpwlen
	if args.pcr:
		pkwargs['crypt'] = args.pcr
	if args.recvs:
		pkwargs['recvs'] = str(args.recvs).split(' ')
	if args.gpgkey:
		pkwargs['gpgkey'] = args.gpgkey
	if args.usr :
		pkwargs['user'] = args.usr
	if args.time:
		pkwargs['time'] = args.time
	if args.yml:
		pkwargs['plain'] = args.yml
	if args.dbg:
		print(bgre(pars))
		print(bgre(tabd(args.__dict__, 2)))
		print(bgre('pargs:\n  %s\npkwargs:\n%s'%(pargs, tabd(pkwargs, 2))))
	if mode != 'gui' and (
          args.yks is False and args.lst is False and \
          args.add is False and args.chg is False and \
          args.rms is False and args.sslcrt is None and \
          args.sslkey is None and args.scl is None):
		pars.print_help()
		exit(0)
	if mode == 'gui':
		return args, pargs, pkwargs
	return args, pargs, pkwargs

def cli():
	args, pargs, pkwargs = confpars('cli')
	if not path.isfile(args.yml) and \
          not path.isfile(args.pcr) and args.yks is False:
		with open(args.yml, 'w+') as yfh:
			yfh.write("""---\n%s:  {}"""%args.usr)
	poclp, boclp = paste('pb')
	if args.yks or args.yks is None:
		if 'YKSERIAL' in environ.keys():
			ykser = environ['YKSERIAL']
		ykser = args.yks if args.yks else None
		if ykser and len(ykser) >= 6:
			ykser = ''.join(str(ykser)[-6:])
		res = ykchalres(getpass(), args.ysl, ykser)
		if not res:
			fatal('could not get valid response on slot ', args.ysl)
		forkwaitclip(res, poclp, boclp, args.time, args.out, args.ent)
		exit(0)
	__ents = {}
	err = None
	if args.add:
		__ents = PassCrypt(*pargs, **pkwargs).adpw(
                           args.add, args.pwd, args.com)
		if not __ents:
			error('something went wrong while adding', args.add)
			exit(1)
		__pc = __ents[args.usr][args.add]
		if __pc:
			if len(__pc) == 2 and osname != 'nt':
				xnotify('%s: %s'%(
                        args.lst, ' '.join(__pc[1:])), __me)
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out, args.ent)
	elif args.chg:
		if args.pwd:
			pkwargs['password'] = args.pwd
		__ents = PassCrypt(*pargs, **pkwargs).chpw(
            args.chg, args.pwd, args.com)
		if not args.aal:
			__ents[args.usr]
		if not __ents:
			if [u for (u, es) in __ents.items() if args.chg in es.keys()]:
				exit(0)
			err = ('could not change entry', args.chg)
	elif args.rms:
		ers = []
		for r in args.rms:
			__ents = PassCrypt(*pargs, **pkwargs).rmpw(r)
			if not args.aal:
				__ents[args.usr]
			if r in __ents.keys():
				ers.append(r)
		ewrd = 'entry'
		if len(ers) >= 1:
			ewrd = 'entrys'
		err = ('deleting the following %s failed:', bred(', ').join(
               ers)) if ers else None
	elif args.lst is not False and args.lst is not None:
		__ents = PassCrypt(*pargs, **pkwargs).lspw(args.lst)
		if __ents and args.lst not in __ents.keys():
			err = (
                'could not find entry', args.lst,
                'for', args.usr, 'in', pkwargs['crypt'])
		elif args.lst and __ents:
			__pc = __ents[args.lst]
			if __pc:
				notif = 'copy'
				if len(__pc) == 2:
					notif = ' '.join(__pc[1:])
				if osname!= 'nt':
					xnotify(notif, __me)
				forkwaitclip(__pc[0], poclp, boclp, args.time, args.out, args.ent)
				exit(0)
	elif args.lst is None:
		__ents = PassCrypt(*pargs, **pkwargs).lspw()
		err = 'no password entrys or decryption failed' if not __ents else None
	elif args.scl:
		t = 0 if 'time' not in pkwargs.keys() else pkwargs['time']
		showclip(args.out if args.out else 'cli', t, True if t else False)
		exit(0)
	if err:
		fatal(err)
	_printpws_(__ents, args.sho)

def __xdialog(msg, sec=None):
	while True:
		rtn = False
		if sec:
			rtn = xgetpass(msg)
		else:
			rtn = xinput(msg)
		if not rtn:
			yesno = xyesno('no input received, try again?')
			if yesno:
				continue
		break
	return rtn

def gui(typ='pw'):
	"""gui wrapper function to not run unnecessary code"""
	poclp, boclp = paste('pb')
	args, pargs, pkwargs = confpars('gui')
	if args.yks or args.yks is None or typ == 'yk':
		res = ykchalres(xgetpass(), args.ysl, args.yks)
		if not res:
			xmsgok('no response from the key (if there is one)'%res)
			exit(1)
		forkwaitclip(res, poclp, boclp, args.time, args.out, args.ent)
	__ents = None
	usr = args.usr
	if args.usr is None:
		usr = __xdialog('enter username for selected action')
	if not usr:
		xnotify('aborted', __me)
		exit(1)
	elif usr == 'all':
		pargs.append('aal')
	else:
		pkwargs['user'] = usr
	if args.add is not False:
		_add = __xdialog('as %s: enter name for entry to add'%usr)
		if not _add:
			xnotify('aborted', __me)
			exit(1)
		if _add:
			__ents = PassCrypt(*pargs, **pkwargs).adpw(_add, None, None)
			ok = True
			if not __ents:
				if 'aal' in pargs:
					for (susr, entrys) in __ents.items():
						if _add not in entrys.keys():
							ok = False
				elif usr not in __ents or _add not in __ents[usr].keys():
					ok = False
			if not ok:
				xnotify('something went wrong while adding %s'%_add, __me)
			__pc = __ents[usr][_add]
			notif = 'copy'
			if len(__pc) == 2:
				notif = ' '.join(__pc[1:])
			if not args.out and osname != 'nt':
				xnotify(notif, __me)
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out, args.ent)
			umsg = 'all users'
			if 'aal' not in pargs:
				umsg = 'user %s'%usr
			xnotify('added entry %s for %s'%(_add, umsg), __me)
	elif args.chg is not False:
		_chg = __xdialog(
            'as %s: enter name of the entry to change'%usr)
		if not _chg:
			xnotify('changing entry aborted', __me)
			exit(1)
		if _chg:
			__ents = PassCrypt(*pargs, **pkwargs).chpw(_chg, None, None)
			if not __ents or usr not in __ents or \
                  _chg not in __ents[usr].keys():
				xnotify('something went wrong while changing %s'%_chg, __me)
				exit(1)
			__pc = __ents[usr][_chg]
			notif = 'copy'
			if len(__pc) == 2:
				notif = ' '.join(__pc[1:])
			if not args.out and osname != 'nt':
				xnotify(notif, __me)
			forkwaitclip(__pc[0], poclp, boclp, args.time, args.out, args.ent)
			xnotify('changed entry %s for %s'%(_chg, usr), __me)
	elif args.rms is not False:
		_rms = __xdialog(
            'as %s: enter name of the entry(s) to delete'%usr)
		if not _rms:
			xnotify('deleting entry aborted', __me)
			exit(1)
		if _rms and ' ' in _rms:
			_rms = [r.strip() for r in _rms.split(' ')]
		else:
			_rms = [_rms]
		for r in _rms:
			pcr = PassCrypt(*pargs, **pkwargs)
			__ents = pcr.rmpw(r)
			pcr.__del__()
			if osname != 'nt':
				if not __ents:
					xnotify('could not delete entry %s'%r, __me)
				else:
					xnotify('deleted entry %s for %s'%(r, usr), __me)
	elif args.lst is not False:
		__in = args.lst
		if not args.lst:
			__in = args.lst
			_umsg = '%s\'s entrys'%usr
			if args.aal:
				_umsg = 'all entrys'
			__in = __xdialog('enter name to search in %s'%_umsg, True)
			if not __in:
				xnotify('aborted', __me)
				exit(1)
		__ent = PassCrypt(*pargs, **pkwargs).lspw(__in)
		if not __ent or (__ent and __in not in __ent.keys() or not __ent[__in]):
			xnotify('either %s or %s where not found in passcrypt'%(
                usr, __in), __me)
			exit(1)
		if __ent:
			__pc = __ent[__in]
			if __pc:
				notif = 'copy'
				if len(__pc) == 2:
					notif = ' '.join(__pc[1:])
				if not args.out and osname != 'nt':
					xnotify(notif, __me)
				forkwaitclip(
                    __pc[0], poclp, boclp, args.time, args.out, args.ent)
				exit(0)
	elif args.scl:
		t = 0 if 'time' not in pkwargs.keys() else pkwargs['time']
		showclip('gui', t, True if t else False)
		exit(0)
	else:
		__ents = PassCrypt(*pargs, **pkwargs).lspw()
	if __ents:
		if args.aal:
			pargs.append('aal')
		if not args.sho:
			__ents = __dctpwreplace(__ents)
		xnotify(tabd(__ents), __me)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
from re import sub
from subprocess import run

def adbenter():
	adb = '/usr/bin/adb'
	run('%s shell input keyevent 66'%adb, shell=True)

def adbout(text, enter=None):
	adb = '/usr/bin/adb'
	text = sub(r'(\s|\$|\%|\&|\(|\)|\=|\?|\-|\!|\+|\.)', r'\\\1', text)
	text = sub(r'"', r'\\\\\"', text)
	text = sub(r"'", r"\'", text)
	run(r'%s shell input text "%s"'%(adb, r'%s'%text), shell=True)
	if enter:
		adbenter()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
Generic Setup script, takes package info from __pkginfo__.py file.
"""
from sys import argv
from os import listdir, chdir, getcwd, path
from setuptools import setup, find_packages

__docformat__ = "restructuredtext en"

def tabl(dats, ind=0, iind=0):
    """list to string with indentations"""
    tabbl = ''
    for i in dats:
        if isinstance(i, (tuple, list)):
            iind = int(max(len(i) for i in dats)+ind)
            tabbl = '%s\n%s'%(tabbl, tabl(i, iind))
            continue
        tabbl = '%s\n%s%s'%(tabbl, ' '*ind, i)
    return tabbl.lstrip('\n')

def tabd(dats, ind=0, iind=0):
    """
    this is a function where i try to guess the best indentation for text
    representation of keyvalue paires with best matching indentation
    e.g:
    foo         = bar
    a           = b
    blibablubb  = bla
    ^^indent "bar" and "b" as much as needed ("add" is added to each length)
    """
    try:
        lim = int(max(len(str(k)) for k in dats if k)+int(ind))
    except ValueError:
        return dats
    tabbd = ''
    try:
        print(dats)
        for (key, val) in dats.items():
            spc = ' '*int(lim-len(str(key)))
            if val and isinstance(val, dict):
                tabbd = '%s\n%s%s:\n%s'%(tabbd, ' '*ind, key, tabd(
                    val, ind+int(iind if iind else 2), iind if iind else 2))
            else:
                tabbd = str('%s\n%s%s%s = %s'%(
                    tabbd, ' '*ind, key, spc, val)).strip('\n')
    except AttributeError:
        return tabl(dats, ind)
    return tabbd.strip('\n')

def packages(directory):
    """return a list of subpackages for the given directory"""
    result = []
    for package in listdir(directory):
        absfile = path.join(directory, package, '__init__.py')
        if path.exists(absfile):
            result.append(path.dirname(absfile))
            result += packages(path.dirname(absfile))
    return result

def scripts(linux_scripts):
    """creates the proper script names required for each platform"""
    from distutils import util
    if util.get_platform()[:3] == 'win':
        return linux_scripts + [script + '.bat' for script in linux_scripts]
    return linux_scripts

if __name__ == '__main__':
    kwargs = {}
    kwargs['packages'] = ['pwclip'] + packages('pwclip')
    with open(path.join('pwclip', '__pkginfo__.py'), 'r') as f:
        __pkginfo = f.read()
        exec(__pkginfo, kwargs)
    if 'long_description' not in kwargs.keys():
        kwargs['long_description'] = """fucker"""
    with open(path.join('pwclip', 'README.md'), 'w+') as rfh:
        rfh.write(str(kwargs['long_description']))
    if '--pybuilder' in argv:
        print(tabd(dict(sorted(kwargs.items()))))
        input('\npress Enter to continue...')
        del argv[argv.index('--pybuilder')]
    try:
        setup(**kwargs)
    except (KeyboardInterrupt, TypeError):
        exit(1)
    exit(0)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""passcrypt module"""

from sys import argv, stdout

from os import path, remove, environ, chmod, stat, makedirs, name as osname

from yaml import load, dump, Loader, Dumper

from colortext import blu, yel, grn, bgre, tabd, error

from system import \
    userfind, filerotate, setfiletime, \
    xgetpass, xmsgok, xinput, xnotify, \
    filerotate, absrelpath, xyesno, random, copy

from secrecy.gpgtools import GPGTool, GPGSMTool, DecryptError, SignatureError

try:
	from atexit import register
except ImportError:
	pass

class PassCrypt(GPGTool):
	"""passcrypt main class"""
	dbg = None
	vrb = None
	aal = None
	fsy = None
	sho = None
	rnd = None
	out = None
	gsm = None
	sig = True
	dtg = True
	gui = None
	chg = None
	syn = None
	try:
		user = userfind()
		home = userfind(user, 'home')
	except FileNotFoundError:
		user = environ['USERNAME']
		home = path.join(environ['HOMEDRIVE'], environ['HOMEPATH'])
	user = user if user else 'root'
	home = home if home else '/root'
	config = path.join(home, '.config', 'pwclip.cfg')
	plain = path.join(home, '.pwd.yaml')
	crypt = path.join(home, '.passcrypt')
	recvs = []
	keys = {}
	gpgkey = ''
	sslcrt = ''
	sslkey = ''
	sigerr = None
	genpwrex = None
	genpwlen = 24
	__weaks = {}
	__oldweaks = {}
	def __init__(self, *args, **kwargs):
		"""passcrypt init function"""
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(PassCrypt.__mro__))
			print(bgre(tabd(PassCrypt.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		self.cpasmsg = '%s %s%s '%(
            blu('enter password for'), yel(self.crypt), blu(':'))
		gargs = list(args) + ['sig'] if self.sig else []
		if self.gui:
			gargs = list(args) + ['gui'] + ['sig'] if self.sig else []
			self.cpasmsg = 'enter password for %s: '%self.crypt
		self.sig = True if 'sig' not in kwargs.keys() else kwargs['sig']
		if self.gsm or (
              self.gsm is None and self.recvs and [
                  r for r in self.recvs if r in GPGSMTool().keylist(True)]):
			GPGSMTool.__init__(self, *args, **kwargs)
		else:
			GPGTool.__init__(self, *args, **kwargs)
		self.keys = self.findkey()
		if not self.keys:
			self._mkconfkeys()
		self._gpgcheck()
		self.__weaks = dict(sorted(dict(self._readcrypt()).items()))
		self.__oldweaks = str(self.__weaks)
		if osname != 'nt':
			register(self._cryptpass)

	def __del__(self):
		self._cryptpass()

	def _cryptpass(self):
		chgs = []
		crecvs = []
		if path.isfile(self.crypt):
			erecvs = list(set(self.recvlist(self.crypt)))
			for r in erecvs:
				crecvs.append('0x%s'%r[-16:])
			for r in self.recvs:
				if r not in crecvs:
					chgs.append('+ %s'%r)
			for r in crecvs:
				if r not in self.recvs:
					chgs.append('- %s'%r)
		if self.__oldweaks != str(dict(sorted(self.__weaks.items()))) or chgs:
			msg = ('recipients have changed:\n',
                    '\n'.join(c for c in chgs),
                    '\nfrom:\n', ' '.join(erecvs),
                    '\nto:\n', ' '.join(self.recvs),
                    '\nencryption enforced...')
			if chgs:
				if not self.gui:
					error(*msg)
				else:
					xmsgok(' '.join(msg))
			if not self.recvs and not self.findkey():
				self.genkeys(**self._gendefs(self.gui))
			if self.__weaks:
				if self._writecrypt(self.__weaks):
					if self.vrb:
						print(blu('file'), yel(self.crypt), blu('encrypted'))

	def _mkconfkeys(self):
		self.gpgkey = '0x%s'%str(self.genkeys())[-16:]
		cfgs = {'gpg': {}}
		override = False
		if path.isfile(self.config):
			override = True
			with open(self.config, 'r') as cfh:
				cfgs = dict(load(cfh.read(), Loader=Loader))
			if 'gpg' not in cfgs.keys():
				cfgs['gpg'] = {}
		cfgs['gpg']['gpgkey'] = self.gpgkey
		if 'recipients' not in cfgs['gpg'].keys():
			cfgs['gpg']['recipients'] = self.gpgkey
			self.recvs = [self.gpgkey]
		with open(self.config, 'w+') as cfh:
			cfh.write(str(dump(cfgs, Dumper=Dumper)))

	def _readcrypt(self):
		"""read crypt file method"""
		if self.dbg:
			print(bgre(self._readcrypt))
		__dct = {}
		try:
			__dct, err = self.decrypt(self.crypt)
		except DecryptError as err:
			error(err)
			exit(1)
		__dct = dict(load(str(__dct), Loader=Loader))
		if err:
			if err == 'SIGERR':
				if self.gui:
					yesno = xyesno('reencrypt, even though ' \
                        'the passcryt signature could not be verified?')
				else:
					print(grn('reencrypt, even though ' \
                        'the passcryt signature could not be verified?'),
                        '[Y/n]')
					yesno = input()
					yesno = True if yesno in ('', 'y') else False
				if yesno and __dct:
					self._writecrypt(__dct)
		return __dct

	def _writecrypt(self, __weaks):
		"""crypt file writing method"""
		if self.dbg:
			print(bgre(self._writecrypt))
		kwargs = {
            'output': self.crypt,
            'gpgkey': self.gpgkey,
            'recvs': self.recvs}
		filerotate(self.crypt, 3)
		if self.sig and self.dtg:
			filerotate('%s.sig'%self.crypt, 3)
		isok = self.encrypt(
            str(dump(__weaks, Dumper=Dumper)), output=self.crypt)
		chmod(self.crypt, 0o600)
		return isok

	def __askpwdcom(self, sysuser, usr, pwd, com, opw=None, ocom=None):
		if self.rnd:
			pwd = self.rndgetpass()
		if self.gui:
			if not pwd:
				pwd = xgetpass(
                    'blank input preserves already set comment/password\nas user %s: enter password for entry %s'%(sysuser, usr))
				pwd = pwd if pwd else opw
				if not pwd:
					xmsgok('password is needed if adding password')
					return
			if not com:
				com = xinput(
				    'enter comment (optional, "___" deletes the comment)')
			com = ocom if not com else com
			if com == '___':
				com = None
		else:
			if not pwd:
				print(
                    blu('blank input preserves already set comment/password'),
                    blu('as user '), yel(sysuser), ': ', sep='')
				pwd = pwd if pwd else self.passwd(msg='%s%s%s%s: '%(
						blu('  enter '), yel('password '),
						blu('for entry '), yel('%s'%usr)))
				pwd = pwd if pwd else opw
				if not pwd:
					error('password is needed if adding password')
					return
			if not com:
				print(
                    blu('  enter '), yel('comment '),
                    blu('(optional, '), yel('___'),
                    blu(' deletes the comment)'), ': ', sep='', end='')
				com = input()
			com = ocom if not com else com
			if com == '___':
				com = None
		return [p for p in [pwd, com] if p is not None]

	def rndgetpass(self):
		while True:
			__pwd = random(self.genpwlen, self.genpwrex)
			yesno = False
			if self.gui:
				yesno = xyesno('use the following password: "%s"?'%__pwd)
			else:
				print('%s %s%s [Y/n]'%(
                    grn('use the following password:'),
                    yel(__pwd), grn('?')), sep='')
				yesno = input()
				yesno = True if str(yesno).lower() in ('y', '') else False
			if yesno:
				break
		return __pwd

	def adpw(self, usr, pwd=None, com=None):
		"""password adding method"""
		if self.dbg:
			print(bgre(tabd({
                self.adpw: {'user': self.user, 'entry': usr,
                            'pwd': pwd, 'comment': com}})))
		if self.user not in self.__weaks.keys():
			self.__weaks[self.user] = {}
		if not self.aal:
			if self.user in self.__weaks.keys():
				if usr in self.__weaks[self.user]:
					if self.gui:
						xmsgok('entry %s already exists for user %s'%(
                            usr, self.user))
					else:
						error(
                            'entry', usr, 'already exists for user', self.user)
					return self.__weaks
				elif self.user not in self.__weaks.keys():
					self.__weaks[self.user] = {}
				try:
					__opw, __ocom = self.__weaks[self.user][usr]
				except KeyError:
					__opw, __ocom = None, None
				pwdcom = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
				if pwdcom:
					self.__weaks[self.user][usr] = [p for p in pwdcom if p]
		else:
			for u in self.__weaks.keys():
				if usr in self.__weaks[u].keys():
					if self.gui:
						xmsgok('entry %s already exists for user %s'%(usr, u))
					else:
						error('entry', usr, 'already exists for user', u)
					continue
				pwdcom = self.__askpwdcom(
                    self.user, usr, pwd, com)
				if pwdcom:
					self.__weaks[u][usr] = [p for p in pwdcom if p]
		return dict(self.__weaks)

	def chpw(self, usr, pwd=None, com=None):
		"""change existing password method"""
		if self.dbg:
			print(bgre(tabd({
                self.chpw: {'user': self.user, 'entry': usr, 'pwd': pwd}})))
		if not self.aal:
			if self.__weaks and self.user in self.__weaks.keys() and \
                    usr in self.__weaks[self.user].keys():
				try:
					__opw = self.__weaks[self.user][usr][0]
				except IndexError:
					return
				try:
					__ocmd = self.__weaks[self.user][usr][1]
				except IndexError:
					__ocmd = None
				self.__weaks[self.user][usr] = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocmd)
			else:
				if self.gui:
					xmsgok('no entry named %s for user %s'%(usr, self.user))
				else:
					error('no entry named', usr, 'for user', self.user)
		else:
			for u in self.__weaks.keys():
				#	if self.gui:
				#		xmsgok('entry %s does not exist for user %s'%(usr, u))
				#	else:
				#		error('entry', usr, 'does not exist for user', u)
				#	continue
				__opw = ''
				__ocom = None
				if usr not in self.__weaks[u].keys():
					self.__weaks[u][usr] = [__opw, __ocom]
				elif self.__weaks[self.user][usr]:
					__opw, __ocom = self.__weaks[self.user][usr]
				self.__weaks[u][usr] = self.__askpwdcom(
                    self.user, usr, pwd, com, __opw, __ocom)
		return dict(self.__weaks)

	def rmpw(self, usr):
		"""remove password method"""
		if self.dbg:
			print(bgre(tabd({self.rmpw: {'user': self.user, 'entry': usr}})))
		if self.aal:
			__w = dict(self.__weaks)
			for u in __w.keys():
				try:
					del self.__weaks[u][usr]
					setattr(self, 'chg', True)
				except KeyError:
					if self.gui:
						xmsgok('entry %s not found as user %s'%(usr, u))
					else:
						error('entry', usr, 'not found as user', u)
				if not self.__weaks[u].keys():
					del self.__weaks[u]
		else:
			if self.user in self.__weaks.keys() and \
                  usr in self.__weaks[self.user].keys():
				del  self.__weaks[self.user][usr]
			else:
				if self.gui:
					xmsgok('entry %s not found as user %s'%(usr, self.user))
				else:
					error('entry', usr, 'not found as user', self.user)
			if self.user in self.__weaks.keys() \
                  and not self.__weaks[self.user].keys():
				del self.__weaks[self.user]
		return dict(self.__weaks)

	def lspw(self, usr=None, aal=None):
		"""password listing method"""
		if self.dbg and not self.gui:
			print(bgre(tabd({self.lspw: {'user': self.user, 'entry': usr}})))
		aal = True if aal else self.aal
		__ents = {}
		if self.__weaks:
			if aal:
				__ents = self.__weaks
				if usr:
					usrs = [self.user] + \
                        [u for u in self.__weaks.keys() if u != self.user]
					for u in usrs:
						if u in self.__weaks.keys() and \
                              usr in self.__weaks[u].keys():
							__ents = {usr: self.__weaks[u][usr]}
							break
			elif self.user in self.__weaks.keys():
				__ents = self.__weaks[self.user]
				if usr in __ents.keys():
					__ents = {usr: self.__weaks[self.user][usr]}
		return dict(__ents)

def lscrypt(usr, dbg=None):
	"""passlist wrapper function"""
	if dbg:
		print(bgre(lscrypt))
	__ents = {}
	if usr:
		__ents = PassCrypt().lspw(usr)
	return __ents




if __name__ == '__main__':
	exit(1)
